import DestinationBean;
import AddDesRec;
import ViewDesRec;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;


public class SmartDirections extends JFrame
{
	// User interface components
	private JEditorPane contents;
	private JPanel leftPanel, leftTopPanel;
	private JList directionList;
	private JTextArea directionText;
	
	// Internal variables for retrieving destination and direction information
	private DestinationBean currentDestination = new DestinationBean();
	private Vector directions;
	private String location, currentDirection = "No details available";
	
	// Objects for adding and viewing destination records
	private static AddDesRec newAdd;
	private static ViewDesRec newView;
	
	// Constructor
	public SmartDirections()
	{
		super("A Direction Guide That Gives Additional Information According to Destinations");
		
		// Menubar
		JMenuBar bar = new JMenuBar();
		setJMenuBar(bar);
		
		// Add items to the menubar and submenus
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');
		
		JMenuItem addDesRecItem = new JMenuItem("Add a New Destination Record...");
		addDesRecItem.setMnemonic('A');
		addDesRecItem.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if (newAdd == null)
				{
					newAdd = new AddDesRec();
				}else newAdd.show();
			}
		}
		);
		fileMenu.add(addDesRecItem);
		
		JMenuItem viewDesRecItem = new JMenuItem("View the Destination Records");
		viewDesRecItem.setMnemonic('V');
		viewDesRecItem.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				newView = new ViewDesRec();
				newView.show();
			
			}
		}
		);
		fileMenu.add(viewDesRecItem);
	
		JMenuItem exitItem = new JMenuItem("Exit");
		exitItem.setMnemonic('x');
		exitItem.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				System.exit(0);
			}
		}
		);
		fileMenu.add(exitItem);
		bar.add(fileMenu);
		
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic('H');
		JMenuItem aboutItem = new JMenuItem("About...");
		aboutItem.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				JOptionPane.showMessageDialog(SmartDirections.this, "This is a demo of the smart direction guide\nIt gives additional information according to destinations.", "About", JOptionPane.PLAIN_MESSAGE);
			}
		}
		);
		helpMenu.add(aboutItem);
		bar.add(helpMenu);
		
		// Set the JPanels
		Container c = getContentPane();
		leftPanel = new JPanel();
		leftPanel.setLayout(new BorderLayout(5,5));
		leftTopPanel = new JPanel();
		leftTopPanel.setLayout(new BorderLayout(5,5));
		JLabel title = new JLabel("Choose a sample direction");
		leftTopPanel.add(title, BorderLayout.NORTH);
		
		// Initialize vector for storing and displaying directions
		directions = new Vector();
		directions.addElement("From home to Galleria shopping");
		directions.addElement("From home to Logan departure");
		directions.addElement("From home to Logan pickup");		
		directionList = new JList(directions);
		directionList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		directionList.addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e)
			{
				currentDestination = getDestinationBean(directionList.getSelectedIndex());		
				getThePage(currentDestination.getRelatedInfo());
				currentDirection = getDirection(directionList.getSelectedIndex());
				directionText.setText(currentDirection);
			}
		}
		);
		
		leftTopPanel.add(directionList, BorderLayout.CENTER);
		leftPanel.add(leftTopPanel, BorderLayout.NORTH);
		directionText = new JTextArea(currentDirection);
		leftPanel.add(new JScrollPane(directionText), BorderLayout.CENTER);
		c.add(leftPanel, BorderLayout.WEST);
		
		// Initialize the component for displaying related web sites
		contents = new JEditorPane();
		contents.setEditable(false);
		contents.addHyperlinkListener(new HyperlinkListener()
		{
			public void hyperlinkUpdate(HyperlinkEvent e)
			{
				try
				{
				if(e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
					getThePage(e.getURL().toString());
				}catch(NullPointerException ne)
				{
				}	
			}
		}
		);
		c.add(new JScrollPane(contents), BorderLayout.CENTER);
		
		setSize(1020,740);
		getThePage("http://web.mit.edu/xxiang/Public/CommonSense/start.html");
		show();
	}

	// Get the web site contents given a URL
	private void getThePage(String location)
	{
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try
		{
			contents.setPage(location);
		}catch(IOException io)
		{
			JOptionPane.showMessageDialog(this, "Error retrieving the web page", "Bad URL", JOptionPane.ERROR_MESSAGE);
		}
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
	
	// Returns the detailed directions given an index
	private String getDirection(int i)
	{
		int c = 0;
		StringBuffer directionStringBuffer = new StringBuffer();
		
		try
		{
			FileInputStream directionStream = new FileInputStream("direction" + i + ".txt");
			while ((c = directionStream.read()) != -1)
			{
				directionStringBuffer.append((char)c);
			}
		}catch (FileNotFoundException e)
		{
			JOptionPane.showMessageDialog(this, "Error retrieving direction"+i+".txt", "File Not Found", JOptionPane.ERROR_MESSAGE);
		}catch (IOException io)
		{
			JOptionPane.showMessageDialog(this, "Error reading direction"+i+".txt", "Error Reading File", JOptionPane.ERROR_MESSAGE);
		}
		return directionStringBuffer.toString();
	}
	
	// Returns a DestinationBean object given an index
	private DestinationBean getDestinationBean(int i)
	{
		String des = "", purpose = "";
		DestinationBean maxLikelihood = new DestinationBean();
		maxLikelihood.setRelatedInfo("http://web.mit.edu/xxiang/Public/CommonSense/no_info.html");
		StringTokenizer st = new StringTokenizer((String)directions.elementAt(i));
		while (st.hasMoreTokens())
		{
			if (st.nextToken().equals("to"))
			{
				des = st.nextToken();
				break;
			}
		}
		if (st.hasMoreTokens())
			purpose = st.nextToken();
		try
		{
			ObjectInputStream input = new ObjectInputStream(new FileInputStream("destination_rec.dat"));
			while (true)
			{
				DestinationBean desBean = (DestinationBean)input.readObject();
				if (desBean.getDestinationName().equalsIgnoreCase(des))
				{
					if (desBean.getPurpose().equalsIgnoreCase(purpose))
					{
						input.close();
						return desBean;
					}
					if ((purpose.equals("")) && (maxLikelihood.getLikelihood() < desBean.getLikelihood()))
						maxLikelihood = desBean;
				}
			}			
		}catch(EOFException eof)
		{
		}catch(ClassNotFoundException cnf)
		{
			JOptionPane.showMessageDialog(null, "Unable to create object", "Class Not Found", JOptionPane.ERROR_MESSAGE);
		}catch(IOException io)
		{
			//JOptionPane.showMessageDialog(null, "Error during read from file", "Read Error", JOptionPane.ERROR_MESSAGE);
		}
		return maxLikelihood;
	}

	public static void main(String args[])
	{
		SmartDirections app = new SmartDirections();
		app.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				if (newAdd != null)
					newAdd.closeFile();
				if (newView != null)
					newView.closeFile();	
				System.exit(0);
			}
		}
		);
	}
}